<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO\Response\Message;
use Teknisa\Libs\Util\Environment;
use Zeedhi\Framework\Report\ReportService;

class CadastroCategoriaSupplyTekController extends \Zeedhi\Framework\Controller\Report {

    protected $environment;
    protected $dataSourceManager;
    protected $crudService;
    protected $newCode;
    protected $entityManager;
    protected $service;
    protected $imgPathURL;
    protected $birtStrategy;
    protected $connection;

    public function __construct ($reportService,$environment, $dataSourceManager, $crudService, $newCode, $entityManager, $service, $imgPathURL,$birtStrategy,$connection) {
    	parent::__construct($reportService);
    	$this->environment = $environment;
    	$this->dataSourceManager = $dataSourceManager;
    	$this->crudService  = $crudService;
    	$this->newCode = $newCode;
        $this->entityManager = $entityManager;
        $this->service = $service;
        $this->imgPathURL = $imgPathURL;
        $this->birtStrategy = $birtStrategy;
        $this->connection  = $connection;
       
    }
    
    public function generateCode($newCode)
    {
        return $this->newCode->getCodeByTableName($newCode, $this->environment->getNrOrgTrab(), 1, 3);
    }
    
    public function saveCategory(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $row['CDCATEGORIA'] = self::generateCode('CATEGORIASUPPLY');
            $this->service->insertNewCategory($row);
            foreach ($row['NMSUBCATEGORIA'] as $subcateg) {
               $row['CDSUBCATEGORIA'] = self::generateCode('SUBCATEGSUPPLY');
               $row['NMSUBCATEGORIA'] = $subcateg;
               $this->service->insertNewSubCategory($row);
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    public function saveChangeProducts($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $prodEx = [];
            foreach ($dataSet as $row) {
                //atualiza o produto
                $this->service->updateProduct($row);
                //se for para inativar
                if($row['IDATIVO'] == 'N'){
                    //verificar se esse produto está em algum carrinho
                    $carrinhos =  $this->service->selectCart($row);
                    foreach ($carrinhos as $carrinho){
                        //verificar se aquele produto já foi excluido no carrinho levando em conta a marca do mesmo.
                        if(!in_array($carrinho['NRPRODCART'].$carrinho['CDMARCA'], $prodEx)){
                            array_push($prodEx, $carrinho['NRPRODCART'].$carrinho['CDMARCA']);
                            $row['CDOPERADOR'] = $carrinho['CDOPERADOR'];
                            $row['CDFILIAL'] = $carrinho['CDFILIAL'];
                            $row['NRPRODCART'] = $carrinho['NRPRODCART'];
                            //notificar o operador
                            $row['DTINCLUSAO'] = date('d/m/Y');
                            $row['IDATIVO'] = 'S';
                            $row['IDNOTIFICLIDO'] = 'N';
                            $row['NMNOTIFICACAO'] = 'AVISO_PRODUTO_EXCLUIDO';
                            $row['DSTEXTNOTIFIC'] = "PRODUTO_INATIVO_SUP";
                            $row['NRNOTIFICSUPPLY'] = str_pad(self::generateCode('NOTIFICSUPPLY') , 10, "0", STR_PAD_LEFT);
                            $this->service->insertNewNotification($row);
                            //remover do carrinho
                            $this->service->deleteProductCart($row);
                        }
                        
                    }
                }

            }
        } catch (\Exception $e) {   
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function productsToInactivate($request, $response) 
    {
        try {
            $row = $request->getRow();
            $prodEx = [];
            $produtos = $this->service->selectProductsToInactivate($row);
            foreach ($produtos as $produto) {
                $produto['IDATIVO'] = 'N';
                $this->service->updateProduct($produto);
                //verificar se esse produto está em algum carrinho
                $carrinhos =  $this->service->selectCart($produto);
                foreach ($carrinhos as $carrinho){
                    //verificar se aquele produto já foi excluido no carrinho
                    if(!in_array($carrinho['NRPRODCART'].$carrinho['CDMARCA'], $prodEx)){
                        array_push($prodEx, $carrinho['NRPRODCART'].$carrinho['CDMARCA']);
                        $produto['CDOPERADOR'] = $carrinho['CDOPERADOR'];
                        $produto['CDFILIAL']   = $carrinho['CDFILIAL'];
                        $produto['NRPRODCART'] = $carrinho['NRPRODCART'];
                        $produto['NMFANTFORN'] = $carrinho['NMFANTFORN'];
                        $produto['NMPRODUTO']  = $carrinho['NMPRODUTO'];
                        $produto['NMOPERADOR'] = $carrinho['NMOPERADOR'];
                        //notificar o operador
                        $produto['DTINCLUSAO'] = date('d/m/Y');
                        $produto['IDATIVO'] = 'S';
                        $produto['IDNOTIFICLIDO'] = 'N';
                        $produto['NMNOTIFICACAO'] = 'AVISO_PRODUTO_EXCLUIDO';
                        $produto['DSTEXTNOTIFIC'] = 'PRODUTO_INATIVO_CATALOGO';
                        $produto['NRNOTIFICSUPPLY'] = str_pad(self::generateCode('NOTIFICSUPPLY') , 10, "0", STR_PAD_LEFT);
                        $this->service->insertNewNotification($produto);
                        //remover do carrinho
                        $this->service->deleteProductCart($produto);
                    }
                    
                }
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function saveNewProduct(DTO\Request\Row $request, DTO\Response $response)
    {
        try {
            $row = $request->getRow();
            $produtos = $this->service->selectCdProduct($row);
            $messages = array();
            
            foreach ($produtos as $produto) {
                $check = false;
                $row["CDPRODUTO"]    = $produto["CDPRODUTO"];
                $row["NMPRODUTO"]    = $produto["NMPRODUTO"];
                $row["CDREGIAO"]     = $produto["CDREGIAO"];
                $row["CDMARCA"]      = $produto["CDMARCA"];
                $row["CDFILIAL"]     = $produto["CDFILIAL"];
                $row["IDATIVO"]      = $produto["IDATIVO"];
                $row["VRPRECUNIITM"] = $produto["VRPRECUNIITM"];

                $registeredProduct = $this->service->checkProductAlreadyRegistered($row);

                if (sizeof($registeredProduct) == 0) {
                    $check = true;
                    $this->service->insertNewProduct($row);
                }

                $this->createMessage($messages, $produto, $registeredProduct, $check);
            }
            if(!empty($messages)){
                sort($messages);
                $response->addDataSet(new DataSet("messages", $messages));
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 1));
        }
    }
    
    private function createMessage(&$message, $product, $registeredProduct, $check)
    {
        switch ($check) {
            case true:
                $message[] = $product["NMPRODUTO"] . " incluído com sucesso. \n";
            break;
            
            case false:
                $message[] = $product["NMPRODUTO"] . " da região " . $registeredProduct["NMREGIAO"] . " não pode ser incluído porque ele já se encontra cadastrado em: " . $registeredProduct["NMCATEGORIA"] . " -> " . $registeredProduct["NMSUBCATEGORIA"] . "\n";
            break;
        }
    }
    
    public function deleteCategory($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $auxSubCat = true;
            $auxCat = true;
            $messages = [];
            // percorre todos as linhas marcadas
            foreach ($dataSet as $row){
                //verifica se é subcategoria
               if($row['CDSUBCATEGORIA'] !== NULL){
                   //verifica se para aquela subcategoria existe algum produto na tabela PRODCART, 
                   //ou seja, se existe alguma compra em andamento para aquela subcategoria
               $subCat = $this->service->verifySubCategory($row);
               }
               //verifica se é categoria
               if($row['PAI'] == NULL){
                //verifica se existem compras para essa categoria
               $cat = $this->service->verifyCategory($row);
               }
               //caso retorne algum dado, significa que existe compra
                if(!empty($subCat) || !empty($cat)){
                    $auxSubCat = false;
                    $auxCat = false;
                    //adiciona mensagem genérica para exibir erro no JS
                     array_push($messages,"error");
                    break;
                }

            }
            //inativa categoria/subcategoria caso a verificação esteja OK
            if($auxSubCat){
                 $categorias = array_filter($dataSet, function($row) {
                    if($row['PAI'] !== NULL) {
                        //$this->service->deleteSubCategory($row);
                        $this->service->inactivateSubCategory($row);
                        $this->service->deleteProdSubCategory($row);
                    }
                    return $row['PAI'] == NULL;
                });
                if(!empty($categorias) && $auxCat) {
                    //$this->service->deleteCategory($categorias);
                    $this->service->inactivateCategory($categorias);
                }
            }
            
            $response->addDataSet(new DataSet("messages", $messages));
           
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteProduct($request, $response)
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $prodEx = [];
             foreach ($dataSet as $row) {
                 //DELETA O PRODUTO NA TABELA PRODSUPPLY
                $this->service->deleteProduct($row);
                //verificar se esse produto está em algum carrinho
                $carrinhos =  $this->service->selectCart($row);
                foreach ($carrinhos as $carrinho){
                    //verificar se aquele produto já foi excluido no carrinho
                    if(!in_array($carrinho['NRPRODCART'].$carrinho['CDMARCA'], $prodEx)){
                        array_push($prodEx, $carrinho['NRPRODCART'].$carrinho['CDMARCA']);
                        $row['CDOPERADOR'] = $carrinho['CDOPERADOR'];
                        $row['CDFILIAL'] = $carrinho['CDFILIAL'];
                        $row['NRPRODCART'] = $carrinho['NRPRODCART'];
                        //notificar o operador
                        $row['DTINCLUSAO'] = date('d/m/Y');
                        $row['IDATIVO'] = 'S';
                        $row['IDNOTIFICLIDO'] = 'N';
                        $row['DTINCLUSAO'] = date('d/m/Y');
                        $row['IDATIVO'] = 'S';
                        $row['IDNOTIFICLIDO'] = 'N';
                        $row['NMNOTIFICACAO'] = 'AVISO_PRODUTO_EXCLUIDO';
                        $row['DSTEXTNOTIFIC'] = 'PRODUTO_INATIVO_SUP';
                        $row['IDATIVO'] = 'S';
                        $row['IDNOTIFICLIDO'] = 'N';
                        $row['NMNOTIFICACAO'] = 'AVISO_PRODUTO_EXCLUIDO';
                        $row['DSTEXTNOTIFIC'] = 'PRODUTO_INATIVO_SUP';
                        $row['NRNOTIFICSUPPLY'] = str_pad(self::generateCode('NOTIFICSUPPLY') , 10, "0", STR_PAD_LEFT);
                        $this->service->insertNewNotification($row);
                        //remover do carrinho
                        $this->service->deleteProductCart($row);
                    }
                    
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function insertSubCatInEdition($request, $response) 
    {
        try {
            $row = $request->getRow();
            foreach ($row['NEW_SUBCAT'] as $subcateg) {
               $row['CDSUBCATEGORIA'] = self::generateCode('SUBCATEGSUPPLY');
               $row['NMSUBCATEGORIA'] = $subcateg['NMSUBCATEGORIA'];
               $this->service->insertNewSubCategory($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }

    public function inactivateSubCategory($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            $messages = [];
            foreach ($dataSet as $row) {
                $subCat = $this->service->verifySubCategory($row);
                if(empty($subCat)){
                    $row['IDATIVO'] = 'N';
                    $this->service->inactivateSubCategory($row);
                }else{
                    array_push($messages,"error");
                }
                
            }
            $response->addDataSet(new DataSet("messages", $messages));
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function updateSubCategory($request, $response) 
    {
        try {
            $dataSet = $request->getDataSet()->getRows();
            foreach ($dataSet as $row) {
                $this->service->updateSubCategory($row);
            }
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function updateCategory($request, $response) 
    {
        try {
            
            $row = $request->getRow();
            $this->service->updateCategory($row);
            
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function selectCategory($request, $response) 
    {
        try {
            $rows = $request->getDataSet()->getRows();
            $retorno = $this->service->selectCategory($rows[0]);
            $response->addDataSet(new DataSet("resp", $retorno));
        } catch (\Exception $e) {
			$response->setError(new Error($e->getMessage(), 500));
		}   
    }
    
    public function produtosSubCategoria(DTO\Request $request, DTO\Response $response) 
    {
		$filterCriteria = $request->getFilterCriteria();
		$filterCriteria->setDataSourceName("grid_produtos_supplyTek");
		$filterCriteria = $this->crudService->getFilterCriteriaWithNrOrg($filterCriteria);
		$newFilterCriteria = $this->crudService->getPopulatedFilterCriteria($filterCriteria, array());
		$dataSet = $this->dataSourceManager->findBy($newFilterCriteria);
		$newDataSet = $this->setImgFullPath($dataSet);
        $response->addDataSet($newDataSet);
	}
	
    private function setImgFullPath($dataSet) 
    {
		$rows = $dataSet->getRows();
		foreach ($rows as &$row) {
			if ($row["DSIDFOTOFILESERV"] !== null) {
				$row["DSIDFOTOFILESERV"] = $this->imgPathURL.$row["DSIDFOTOFILESERV"];
			}
		}
		return new DataSet("grid_produtos_supplyTek", $rows);
	}

     protected $reportMapping = array (
       'CATALOGO_PRODUTO_UNIDADE' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        'P_CDFILIAL'       =>'P_CDFILIAL',
                        'P_CDFORNECED'    =>'P_CDFORNECED',
                        'P_CDCATEGORIA'   =>'P_CDCATEGORIA',
                        'P_CDSUBCATEGORIA'=>'P_CDSUBCATEGORIA',
                        'P_CDREGIAO'      =>'P_CDREGIAO',
                        'CDARVPRODINI'  =>'P_CDARVPRODINI',
                        'CDARVPRODFIN'  =>'P_CDARVPRODFIN',
                        'P_TCDARVPROD'  =>'P_TCDARVPROD',
                        'IDCONSPROD'    =>'P_IDCONSPROD',
                        'NRORG'         =>'P_NRORG',
                    ),
            ),
         'CATALOGO_PRODUTO_FOTO' =>
            array (
                'strategy' => 'BIRT',
                'parameterMapping' =>
                    array (
                        
                        'P_CDFILIAL'      =>'P_CDFILIAL',
                        'P_CDFORNECED'    =>'P_CDFORNECED',
                        'P_CDCATEGORIA'   =>'P_CDCATEGORIA',
                        'P_CDSUBCATEGORIA'=>'P_CDSUBCATEGORIA',
                        'P_CDREGIAO'      =>'P_CDREGIAO',
                        'CDARVPRODINI'    =>'P_CDARVPRODINI',
                        'CDARVPRODFIN'    =>'P_CDARVPRODFIN',
                        'P_TCDARVPROD'    =>'P_TCDARVPROD',
                        'IDCONSPROD'      =>'P_IDCONSPROD',
                        'URL'             =>'P_URL',
                        'NRORG'           =>'P_NRORG',
                    ),
            ),
        );
    
     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        
        if($row['CDARVPRODINI'] == ""){
            $row['NMPRODINI'] = "";
        }
         if($row['P_CDCATEGORIA'] == ""){
             $row['P_CDCATEGORIA'] ='T';
         }else{
             $nrlistaForn = $this->criarListaValores($row['P_CDCATEGORIA']);
             $row['P_CDCATEGORIA'] = "_NRLISTA=".$nrlistaForn;
         }
         if($row['P_CDSUBCATEGORIA'] == "" ){
            $row['P_CDSUBCATEGORIA']  ='T';
        }else{
             $nrlistaForn = $this->criarListaValores($row['P_CDSUBCATEGORIA']);
             $row['P_CDSUBCATEGORIA'] = "_NRLISTA=".$nrlistaForn;
         }
         if($row['P_CDFILIAL'] == ""){
            $row['P_CDFILIAL'] ='T';
        }else{
             $nrlistaForn = $this->criarListaValores($row['P_CDFILIAL']);
             $row['P_CDFILIAL'] = "_NRLISTA=".$nrlistaForn;
         }
         if($row['P_CDFORNECED'] == ""){
              $row['P_CDFORNECED'] ='T';
        }else{
             $nrlistaForn = $this->criarListaValores($row['P_CDFORNECED']);
             $row['P_CDFORNECED'] = "_NRLISTA=".$nrlistaForn;
         }
         if($row['P_CDREGIAO'] == ""){
            $row['P_CDREGIAO']  ='T';
        }else{
             $nrlistaForn = $this->criarListaValores($row['P_CDFORNECED']);
             $row['P_CDFORNECED'] = "_NRLISTA=".$nrlistaForn;
         }
        if($row['NMPRODINI'] == ""){
            $row['CDARVPRODINI']=null;
            $row['CDARVPRODFIN']=null;
            $row['P_TCDARVPROD']='T';
        }else{
             $row['P_TCDARVPROD']=null;
        }
        $row['NRORG'] = $this->environment->getNrOrg();
        $row["URL"]=$this->imgPathURL;
    }
    
    protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
     }

     public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );

        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
}