<?php
/**
 Quando o local de emissão precisa ser atualizado no processo de apuração do faturamento.
 */
namespace Faturamento\Logic\ApuracaoFaturamento\Exception;

class AtualizaLocalEmissao extends \Faturamento\Logic\Exception
{

    private $messageName;

    /**
     * Construtor.
     *
     * Recebe o nome da mensagem cadastrada no setforms.
     *
     * @param string $messageName
     * @param string $message
     * @param int $code
     * @param \Exception|null $previous
     */
    public function __construct($messageName, $params = null, $code = 0, \Exception $previous = null)
    {
        parent::__construct($messageName, $code, $previous);
        $this->messageName = $messageName;
    }


    /**
     * Retorna o nome da mensagem cadastrada no Setforms para esta excecao.
     *
     * @return string
     */
    public function getMessageName()
    {
        return $this->messageName;
    }


    /**
     * Informa o local de emissao no apuracao do faturamento.
     * @static
     * @throws LocalEmissao
     */
    public static function informaLocalEmissao()
    {
        throw new self (tk_get_message_by_name("fai0800_informa_local_emissao"));
    }


    /**
     * Notas fiscais não foram geradas no período informado
     * @static
     * @throws AtualizaLocalEmissao
     */
    public static function NotaFiscalNaoGerada()
    {
        throw new self (tk_get_message_by_name("fai00800_nota_fiscal_nao_gerada"));
    }


    /**
     * Nº CDFR não foi gerada, pois não há intervalo determinado para o local de emissão.
     * @static
     * @throws AtualizaLocalEmissao
     */
    public static function CdfrNaoGeradaIntervaloIndeterminado()
    {
        throw new self (tk_get_message_by_name("fai00800_cdfr_nao_gerada_intervalo_indeterm_localemis"));
    }


    /**
     * Nº CDFR não foi gerada, pois não há numeração disponível no intervalo determinado para o local de emissão.
     * @static
     * @throws AtualizaLocalEmissao
     */
    public static function CdfrNaoGeradaNumeracaoIndisponivel()
    {
        throw new self (tk_get_message_by_name("fai00800_cdfr_nao_gerada_intervalo_indeterm_localemis"));
    }

}